SystemOrganization addCategory: #GODBases!

GCConference subclass: #GDBConference
	instanceVariableNames: 'conferenceId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBConference commentStamp: 'lacm 11/10/2014 16:10' prior: 0!
A GDBConference is a temporary class that simulates a GCConference class.
!

!GDBConference methodsFor: 'as yet unclassified' stamp: 'lacm 11/10/2014 15:33'!
conferenceId
	^conferenceId! !

!GDBConference methodsFor: 'as yet unclassified' stamp: 'lacm 11/10/2014 15:33'!
conferenceId: anInteger
	conferenceId:= anInteger! !

GODData subclass: #GDBController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBController commentStamp: 'lacm 11/10/2014 17:45' prior: 0!
A GDBController is a main class that permits differents transactions in our Magma database.

1.-	To create the repository and connect the server, run only once the following line in a workspace:

		GDBController startServer

	Then, save it as an image server. To close the server, close that image saved, and when you open it again, the server will already be running.

2.-	Before following these instructions, first set up your server in a separate running image by following the instructions in the item 1.
	After you have the server running, switch back to this image and do the following.
	
	2.1.-  First, run the following:	
			GDBController uniqueInstance.
	
	2.2.- Then, you can make different transactions in the database, such as:
		 
		2.2.1.- 	You may add data in the GODData run the following:
					GDBController add:  myGODData.
		  		where, myGODData is an GODData object.
				
		2.2.2.- 	You may remove a GODData in the Magma database run the following:
					GDBController remove:  myGODData.

		2.2.3.- 	You may make an exact search by author or title in a GODData stored in the database running the following:
					GDBController searchAuthor: 'anAuthorName'.	
						or
					GDBController searchTitle: 'aTitleName'.	

	2.3.-  To refresh the client session for the same client run the following:
			GDBController refreshSessionClient.
			
	2.4.-  To refresh the client session for other client run the following:
			GDBController refreshSessionClient: 'anOtherClientName'.
			
	2.5.-  Finally, to release the client run the following:
			GDBController release.
		!

!GDBController class methodsFor: 'adding' stamp: 'lacm 11/10/2014 15:01'!
add: aGODData
	"This method permits to insert a GODData object in the datase"

      |myReader|

      (aGODData class = GODData) ifFalse:[GDBDataMustBeAnGODData signal].
      (aGODData author isNil) ifTrue:[GODDataAuthorMustContainsSomeInformation signal].
      (aGODData title isNil) ifTrue:[GODDataTitleMustContainsSomeInformation signal].
      GDBMagmaClient commit: [GDBConnection database add:  (self updateId: aGODData)].	
	
	Transcript show: (aGODData,' object was added!!!!'); cr; yourself.
	myReader := GDBConnection database where:[:reader| reader read: #id from: 1]  distinct: true sortBy: #id descending: true.		
	myReader do: [:each| Transcript show: 'id: ',each id; show: '  author: '; show: each author; show: '  title: '; show: each title; cr; yourself.].
! !

!GDBController class methodsFor: 'adding' stamp: 'lacm 11/10/2014 17:38'!
addConference: aGCConference
	"This method permits to insert a GCConference object in the datase"
      |myReader|

"      (aGCConference superclass = GCConference) ifFalse:[GDBConferenceDataMustBeAnGCConference signal]." 
"      (aGCConference identifier isNil) ifTrue:[GCConferenceIdentifierMustContainsSomeInformation signal]."
"      (aGCConference name isNil) ifTrue:[GCConferenceNameMustContainsSomeInformation signal]."
"      (aGCConference acronym isNil) ifTrue:[GCConferenceAcronymMustContainsSomeInformation signal]."
"      (aGCConference edition isNil) ifTrue:[GCConferenceEditionMustContainsSomeInformation signal]."
      GDBMagmaClient commit: [GDBConnection databaseConference add:  (self updateIdConference: aGCConference)].
	
	Transcript show: (aGCConference,' object was added!!!!'); cr; yourself.
	myReader := GDBConnection databaseConference where:[:reader| reader read: #conferenceId from: 1]  distinct: true sortBy: #conferenceId descending: true.
	myReader do: [:each| Transcript show: ' Conference Id:  ',each conferenceId; 
									 show: ' Identifier:  ',each identifier; 
									 show: ' Name:  ', each name; 
									 show: ' Acronym:  ', each acronym; 
									 show: ' Edition:  ',each edition;
									cr; yourself.].
! !

!GDBController class methodsFor: 'repository' stamp: 'lacm 11/8/2014 12:49'!
createRepository	
	"This method permits to create a repository (i.e., the database)"
	GDBMagmaServer new createGDBRepository.! !

!GDBController class methodsFor: 'repository' stamp: 'lacm 11/8/2014 11:55'!
deleteRepository	
	"This method permits to delete the repository (i.e., the database)"
	GDBMagmaServer new deleteGDBRepository.! !

!GDBController class methodsFor: 'searching' stamp: 'TD 11/9/2014 17:47'!
getAll   
	"This method gets all element of the data base."
	| myReader|	
	myReader := GDBConnection database.
	
	^myReader! !

!GDBController class methodsFor: 'client' stamp: 'lacm 11/8/2014 13:38'!
refreshSessionClient
	"This method permits to refresh a client session"
	|aclient|
	aclient:=GDBMagmaClient aClientName. 
	self release.
	GDBMagmaClient aClientName: aclient. 
	GDBMagmaClient session refresh.
! !

!GDBController class methodsFor: 'client' stamp: 'lacm 11/8/2014 13:32'!
refreshSessionClient: aClientName
	"This method permits to refresh a client session, where aclientName is the client name"
	self release.
      GDBMagmaClient aClientName: aClientName.
	GDBMagmaClient session refresh.
! !

!GDBController class methodsFor: 'initialize-release' stamp: 'lacm 11/1/2014 01:00'!
release
	GDBMagmaClient release.! !

!GDBController class methodsFor: 'removing' stamp: 'lacm 11/8/2014 11:55'!
remove: aGODData
	"This method permits to delete a GODData object in the datase"
	|myReader|
       GDBMagmaClient commit: [GDBConnection database remove: aGODData].
	Transcript show: aGODData ,' object was removed!!!!'; cr.
	myReader := GDBConnection database where:[:reader| reader read: #id from: 1]  distinct: true sortBy: #id descending: true.		
	myReader do: [:each| Transcript show: 'id: ',each id; show: '  author: '; show: each author; show: '  title: '; show: each title; cr; yourself.].
! !

!GDBController class methodsFor: 'removing' stamp: 'lacm 11/10/2014 15:36'!
removeConference: aGCConference
	"This method permits to delete a GCConference object in the datase"
	|myReader|
       GDBMagmaClient commit: [GDBConnection databaseConference remove: aGCConference].
	Transcript show: aGCConference ,' object was removed!!!!'; cr.
	
	myReader := GDBConnection databaseConference where:[:reader| reader read: #conferenceId from: 1]  distinct: true sortBy: #conferenceId descending: true.
	myReader do: [:each| Transcript show: ' Conference Id:  ',each conferenceId; 
									 show: ' Identifier:  ',each identifier; 
									 show: ' Name:  ', each name; 
									 show: ' Acronym:  ', each acronym; 
									 show: ' Edition:  ',each edition;
									cr; yourself.].
! !

!GDBController class methodsFor: 'searching' stamp: 'lacm 11/8/2014 11:53'!
searchAuthor: aString    
	"This method permits an exact search by athor."
	| myReader|	
	myReader := GDBConnection database where:[:reader| reader author at: aString].
	
	(myReader size = 0)	
		ifFalse:[Transcript show:' The author ', aString,' is in this database!!.'; cr.
				myReader do: [:each| Transcript show: 'id: ',each id; show: '  author: '; show: each author; show: '  title: '; show: each title; cr; yourself.]
		];
		ifTrue:[	Transcript show:' The author ', aString,' not is in this database.'; cr.].
	
	^myReader! !

!GDBController class methodsFor: 'searching' stamp: 'lacm 11/10/2014 14:40'!
searchFor: aBlock    
	"This method searches for elements that satisfies aBlock conditions"
	| aBag collection|	
	collection :=  GDBController getAll.
	aBag := Bag new.
	collection do: [:a| (aBlock value: a) ifTrue: [ aBag add: a.] ].
	^aBag.! !

!GDBController class methodsFor: 'searching' stamp: 'lacm 11/8/2014 11:53'!
searchTitle: aString    
	"This method permits an exact search by title."
	| myReader|	
	myReader := GDBConnection database where:[:reader| reader title at: aString].		
	(myReader size = 0)	
		ifFalse:[Transcript show:' The title ', aString,' is in this database!!.'; cr.
				myReader do: [:each| Transcript show: 'id: ',each id; show: '  author: '; show: each author; show: '  title: '; show: each title; cr; yourself.].
		];
		ifTrue:[	Transcript show:' The title ', aString,' not is in this database.'; cr.].
	
	^myReader! !

!GDBController class methodsFor: 'client' stamp: 'lacm 11/8/2014 12:00'!
sessionClient
	GDBMagmaClient session.
	! !

!GDBController class methodsFor: 'server' stamp: 'lacm 11/8/2014 11:56'!
startServer
	"This method permits to start the server"
	GDBMagmaServer new startGDBServer.! !

!GDBController class methodsFor: 'server' stamp: 'lacm 11/8/2014 12:31'!
stopServer
	"This method permits to stop the server"
	GDBMagmaServer new stopGDBServer.! !

!GDBController class methodsFor: 'initialize-release' stamp: 'lacm 11/8/2014 12:47'!
uniqueInstance
	"This method permits to initialize a session client and create a MagmaCollection to save GODData objects in the repository (i.e. our database)."
      GDBConnection connection.
! !

!GDBController class methodsFor: 'adding' stamp: 'lacm 11/8/2014 11:57'!
updateId: aGODData
	"this method updates the GODData id respect to the database"

	|newGODData newId myReader|	
	myReader := GDBConnection database where: [:reader | reader read: #id from: 1].
	(myReader size = 0) 
		ifTrue: [newId :=1];
		ifFalse: [newId:= (myReader detectMax: [:each | each id]) id +1].
	newGODData:= aGODData id: newId.
	^newGODData! !

!GDBController class methodsFor: 'adding' stamp: 'lacm 11/10/2014 15:54'!
updateIdConference: aGCConference
	"this method updates the GCConference id respect to the database"
	
	|newGCConference newId myReader|	

	myReader := GDBConnection databaseConference where: [:reader | reader read: #conferenceId from: 1].
	(myReader size = 0) 
		ifTrue: [newId :=1];
		ifFalse: [newId:= (myReader detectMax: [:each | each conferenceId]) conferenceId + 1].
	newGCConference:= aGCConference conferenceId: newId. 
	^newGCConference! !

Object subclass: #GDBConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBConnection commentStamp: 'lacm 11/8/2014 13:20' prior: 0!
A GDBConnection is a class that content o method "database", that permit a connection of the "client" with the database.

!

!GDBConnection class methodsFor: 'connection' stamp: 'lacm 11/10/2014 15:34'!
connection
      GDBMagmaClient session.          
	GDBMagmaClient commit: [
		GDBMagmaClient root at: 'gdbdata'
							  put: (MagmaCollection new		
								addIndex:(MaIntegerIndex attribute: #id);	
								addIndex:(MaSearchStringIndex attribute: #author);		
								addIndex:(MaSearchStringIndex attribute: #title);		
								yourself).
		GDBMagmaClient root at: 'gdbconference'
							  put: (MagmaCollection new		
								addIndex:(MaIntegerIndex attribute: #conferenceId);
								addIndex:(MaSearchStringIndex attribute: #identifier);														
								addIndex:(MaSearchStringIndex attribute: #name);
								addIndex:(MaSearchStringIndex attribute: #acronym);
								addIndex:(MaIntegerIndex attribute: #edition);	
								yourself).
		].
	
	GDBMagmaClient root.
! !

!GDBConnection class methodsFor: 'connection' stamp: 'lacm 11/1/2014 00:37'!
database
      "we create a new key for the root dictionary"
	^ GDBMagmaClient root at: 'gdbdata'.! !

!GDBConnection class methodsFor: 'connection' stamp: 'lacm 11/10/2014 14:06'!
databaseConference
      "we create a new key for the root dictionary"
	^ GDBMagmaClient root at: 'gdbconference'.! !

Object subclass: #GDBMagmaClient
	instanceVariableNames: 'session'
	classVariableNames: 'DefaultConnection'
	poolDictionaries: ''
	category: 'GODBases'!

GDBMagmaClient class
	instanceVariableNames: 'aClientName'!

!GDBMagmaClient commentStamp: 'lacm 11/8/2014 13:19' prior: 0!
A GDBMagmaClient is a class that permits to initialize a client session.!

GDBMagmaClient class
	instanceVariableNames: 'aClientName'!

!GDBMagmaClient class methodsFor: 'private' stamp: 'lacm 11/8/2014 12:38'!
aClientName
	^aClientName! !

!GDBMagmaClient class methodsFor: 'private' stamp: 'lacm 11/8/2014 12:38'!
aClientName: aString
	aClientName:= aString! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 9/28/2014 20:53'!
commit: aBlock
	self default session commit: aBlock! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 11/8/2014 12:36'!
default
	^ DefaultConnection ifNil: [
			DefaultConnection := self new connect: aClientName; 
			yourself
			].! !

!GDBMagmaClient class methodsFor: 'initialize-release' stamp: 'lacm 11/8/2014 12:37'!
initialize
	aClientName:= 'lucy_'! !

!GDBMagmaClient class methodsFor: 'initialize-release' stamp: 'lacm 10/15/2014 12:21'!
release
	(DefaultConnection isNil)
	ifFalse:[
			DefaultConnection session disconnectAndClose.
			DefaultConnection := nil.
		]! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 11/1/2014 01:24'!
root
"This method retrieves the connection for access to the session, and then access to the root method of Magma "
	^ self session root! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 11/1/2014 01:24'!
session
	^ self default session! !

!GDBMagmaClient methodsFor: 'connection' stamp: 'lacm 11/8/2014 12:27'!
connect
	session := (MagmaRemoteLocation
					host: 'localhost'
					port: 51001) newSession.
	session connectAs: 'lucy'.! !

!GDBMagmaClient methodsFor: 'connection' stamp: 'lacm 11/8/2014 12:27'!
connect: aString
	session := (MagmaRemoteLocation
					host: 'localhost'
					port: 51001) newSession.
	session connectAs: aString.! !

!GDBMagmaClient methodsFor: 'private' stamp: 'lacm 9/28/2014 20:51'!
session
	^ session! !

!GDBMagmaClient methodsFor: 'private' stamp: 'lacm 10/15/2014 12:23'!
session: anObject
	session := anObject! !

Object subclass: #GDBMagmaServer
	instanceVariableNames: 'gdbServer repositoryName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBMagmaServer commentStamp: 'lacm 11/8/2014 13:18' prior: 0!
A GDBMagmaServer is a class that permits start and stop the server.!

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/3/2014 16:09'!
createFullbackupTo: aFileDirectory	
	|openControllers|	
	openControllers := MagmaRepositoryController openControllers.	
	openControllers isEmpty	
		ifTrue:[self inform: 'Please start database first'. ^nil].	
	(openControllers size > 1)
		ifTrue: [self inform: 'More than one openController found'. ^nil.].	
	openControllers do: [:aCtrl|
						aCtrl fullBackupTo:					
							(FileDirectory on: aFileDirectory)].! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/3/2014 16:05'!
createGDBRepository
self isCreated
      ifFalse:
	[MagmaRepositoryController
		create: self repositoryName
		root:  Dictionary new].! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/8/2014 11:18'!
deleteGDBRepository
	self isCreated      
		ifTrue:	[GDBController release.	
				 self stopGDBServer.	
				MagmaRepositoryController		
						delete: self repositoryName.].! !

!GDBMagmaServer methodsFor: 'initialize-release' stamp: 'lacm 11/8/2014 11:22'!
initialize	
	repositoryName:= 'repositoryExample'! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/3/2014 16:03'!
isCreated	
	^(FileDirectory forFileName: '') directoryExists: self repositoryName! !

!GDBMagmaServer methodsFor: 'private' stamp: 'lacm 10/15/2014 12:09'!
port
	^ 51001! !

!GDBMagmaServer methodsFor: 'private' stamp: 'lacm 11/8/2014 11:23'!
repositoryName
	^  repositoryName! !

!GDBMagmaServer methodsFor: 'private' stamp: 'lacm 11/8/2014 11:23'!
repositoryName: aRepositoryName
	 repositoryName:= aRepositoryName.! !

!GDBMagmaServer methodsFor: 'server' stamp: 'lacm 11/3/2014 16:45'!
startGDBServer
	self createGDBRepository.
	self stopGDBServer.
	gdbServer := MagmaServerConsole new
		open: self repositoryName;
		processOn: self port;
		yourself.
	self inspect! !

!GDBMagmaServer methodsFor: 'server' stamp: 'lacm 11/5/2014 15:11'!
stopGDBServer
	MagmaServerConsole allInstancesDo:
		[:aServerConsole| aServerConsole shutdown]! !
GDBMagmaClient initialize!
